from distutils.core import setup

setup(
    name='Grew',
    version='0.1.1',
    packages=['grew',],
    license='LICENSE/Licence_CeCILL_V2-en.txt',
    long_description=open('README').read(),
    url="http://pypi.python.org/pypi/Grew",
    author="bguil",
    author_email="Bruno.Guillaume@loria.fr"
)
